==> wiki3_5_9.zip: wiki3_5_9/plugin/attach/AttachHandler.pm <==
############################################################
#
# źեեΥϥɥ顣
#
############################################################
package plugin::attach::AttachHandler;
use strict;
#===========================================================
# 󥹥ȥ饯
#===========================================================
sub new {
	my $class = shift;
	my $self = {};
	return bless $self,$class;
}

#===========================================================
# μ¹
#===========================================================
sub do_action {
	my $self = shift;
	my $wiki = shift;
	my $cgi = $wiki->get_CGI;
	
	my $pagename = $cgi->param("page");
	if($pagename eq ""){
		$pagename = $wiki->config("frontpage");
	}
	
	$wiki->set_title("եź",1);
	
	if($cgi->param("UPLOAD") ne "" || $cgi->param("CONFIRM") ne "" || $cgi->param("DELETE")){
		if(!$wiki->can_modify_page($pagename)){
			return $wiki->error("Խ϶ػߤƤޤ");
		}
	}
	
	#-------------------------------------------------------
	# åץɼ¹
	if($cgi->param("UPLOAD") ne ""){
		my $filename = $cgi->param("file");
		$filename =~ s/\\/\//g;
		$filename = substr($filename,rindex($filename,"/")+1);
		&Jcode::convert(\$filename,'euc');
		
		if($filename eq ""){
			return $wiki->error("ե뤬ꤵƤޤ");
		}
		
		my $hundle   = $cgi->upload("file");
		my $filecont;
		while(<$hundle>){ $filecont = $filecont.$_; }
		
		my $uploadfile = $wiki->config('attach_dir')."/".&Util::url_encode($pagename).".".&Util::url_encode($filename);
		
		open(DATA,">$uploadfile") or die $!;
		binmode(DATA);
		print DATA $filecont;
		close(DATA);
				
		# attachץ饰󤫤źդ줿
		if(defined($cgi->param("count"))){
			my @lines = split(/\n/,$wiki->get_page($pagename));
			my $flag = 0;
			my $form_count = 1;
			my $count=$cgi->param("count");
			my $content = "";
			foreach(@lines){
				if(index($_," ")==0||index($_,"\t")==0){
					$content .= $_."\n";
					next;
				}
				if(index($_,"{{attach}}")!=-1 && $flag==0){
					if($form_count==$count){
						$content = $content."{{ref ".$filename."}}\n";
						$flag = 1;
					} else {
						$form_count++;
					}
				}
				$content = $content.$_."\n";
			}
			if($flag==1){
				$wiki->save_page($pagename,$content);
			}
		}
		
		# εϿ
		&write_log($wiki,"UPLOAD",$pagename,$filename);
		
		$wiki->redirect($pagename);
		
	#-------------------------------------------------------
	# ǧ
	} elsif($cgi->param("CONFIRM") ne ""){
		my $file = $cgi->param("file");
		if($file eq ""){
			return $wiki->error("ե뤬ꤵƤޤ");
		}
		
		my $buf = "";
		
		$buf .= "<a href=\"".$wiki->config('script_name')."?page=".&Util::url_encode($pagename)."\">".
		        $cgi->escapeHTML($pagename)."</a>".$cgi->escapeHTML($file)."ƤǤ\n".
		        "<form action=\"".$wiki->config('script_name')."\" method=\"POST\">\n".
		        "  <input type=\"submit\" name=\"DELETE\" value=\" \">".
		        "  <input type=\"hidden\" name=\"action\" value=\"ATTACH\">".
		        "  <input type=\"hidden\" name=\"page\" value=\"".$cgi->escapeHTML($pagename)."\">".
		        "  <input type=\"hidden\" name=\"file\" value=\"".$cgi->escapeHTML($file)."\">".
		        "</form>";
		return $buf;
	
	#-------------------------------------------------------
	# ¹
	} elsif($cgi->param("DELETE") ne ""){
		my $file = $cgi->param("file");
		if($file eq ""){
			return $wiki->error("ե뤬ꤵƤޤ");
		}
		
		# εϿ
		&write_log($wiki,"DELETE",$pagename,$file);

		unlink($wiki->config('attach_dir')."/".&Util::url_encode($pagename).".".&Util::url_encode($file));
		$wiki->redirect($pagename);
		
	#-------------------------------------------------------
	# 
	} else {
		my $file = $cgi->param("file");
		if($file eq ""){
			return $wiki->error("ե뤬ꤵƤޤ");
		}
		if(!$wiki->can_show($pagename)){
			return $wiki->error("ڡλȸ¤ޤ");
		}
		
		my $contenttype = &get_mime_type($wiki,$file);
		my $ua = $ENV{"HTTP_USER_AGENT"};
		my $disposition = ($contenttype =~ /^image\// && $ua !~ /MSIE/ ? "inline" : "attachment");
		print "Content-Type: $contenttype\n";
		print "Content-Disposition: $disposition;filename=\"".&Jcode::convert($file,'sjis')."\"\n\n";
		open(DATA,$wiki->config('attach_dir')."/".&Util::url_encode($pagename).".".&Util::url_encode($file)) or die $!;
		binmode(DATA);
		while(<DATA>){ print $_; }
		close(DATA);
				
		# εϿ
		&write_log($wiki,"DOWNLOAD",$pagename,$file);
		&count_up($wiki,$pagename,$file);
		
		$wiki->exit();
	}
}

#===========================================================
# ɥȤ򥤥󥯥
#===========================================================
sub count_up {
	my $wiki = shift;
	my $page = shift;
	my $file = shift;
	
	my $count = Util::load_config_hash(undef,$wiki->config('log_dir')."/".$wiki->config('download_count_file'));
	unless(defined($count->{$page."::".$file})){
		$count->{$page."::".$file} = 1;
	} else {
		$count->{$page."::".$file}++;
	}
	Util::save_config_hash(undef, $wiki->config('log_dir')."/".$wiki->config('download_count_file'), $count);
}

#===========================================================
# źեեΥ
#===========================================================
sub write_log(){
	my $wiki = shift;
	my $mode = shift;
	my $page = shift;
	my $file = shift;
	if($wiki->config('log_dir') eq "" || $wiki->config('attach_log_file') eq ""){
		return;
	}
	my $ip  = $ENV{"REMOTE_ADDR"};
	my $ref = $ENV{"HTTP_REFERER"};
	my $ua  = $ENV{"HTTP_USER_AGENT"};
	if($ip  eq ""){ $ip  = "-"; }
	if($ref eq ""){ $ref = "-"; }
	if($ua  eq ""){ $ua  = "-"; }
	my ($sec, $min, $hour, $mday, $mon, $year) = localtime(time());
	my $date = sprintf("%04d/%02d/%02d %02d:%02d:%02d",$year+1900,$mon+1,$mday,$hour,$min,$sec);
	
	open(LOG,">>".$wiki->config('log_dir')."/".$wiki->config('attach_log_file')) or die $!;
	binmode(LOG);
	print LOG $mode." ".&Util::url_encode($page)." ".&Util::url_encode($file)." ".
	          $date." ".$ip." ".$ref." ".$ua."\n";
	close(LOG);
}

#===========================================================
# MIMEפޤ
#===========================================================
sub get_mime_type {
	my $wiki = shift;
	my $file = shift;
	my $type = lc(substr($file,rindex($file,".")+1));
	
	my $hash  = &Util::load_config_hash($wiki,"mime.dat");
	my $ctype = $hash->{$type};
	
	if($ctype eq "" ){
		$ctype = "application/octet-stream";
	}
	
	return $ctype;
}

1;

==> wiki3_5_11.zip: wiki3_5_11/plugin/attach/AttachHandler.pm <==
############################################################
#
# źեեΥϥɥ顣
#
############################################################
package plugin::attach::AttachHandler;
use strict;
use plugin::attach::Files;
#===========================================================
# 󥹥ȥ饯
#===========================================================
sub new {
	my $class = shift;
	my $self = {};
	return bless $self,$class;
}

#===========================================================
# μ¹
#===========================================================
sub do_action {
	my $self = shift;
	my $wiki = shift;
	my $cgi = $wiki->get_CGI;
	
	my $pagename = $cgi->param("page");
	if($pagename eq ""){
		$pagename = $wiki->config("frontpage");
	}
	
	$wiki->set_title("եź",1);
	
	if($cgi->param("UPLOAD") ne "" || $cgi->param("CONFIRM") ne "" || $cgi->param("DELETE") ne ""){
		unless($wiki->can_modify_page($pagename)){
			return $wiki->error("Խ϶ػߤƤޤ");
		}
	}
	
	if($cgi->param("DELETE") ne ""){
		unless(&plugin::attach::Files::can_attach_delete($wiki, $pagename)){
			return $wiki->error("եκϵĤƤޤ");
		}
	}
	
	#-------------------------------------------------------
	# åץɼ¹
	if($cgi->param("UPLOAD") ne ""){
		my $filename = $cgi->param("file");
		$filename =~ s/\\/\//g;
		$filename = substr($filename,rindex($filename,"/")+1);
		&Jcode::convert(\$filename,'euc');
		
		if($filename eq ""){
			return $wiki->error("ե뤬ꤵƤޤ");
		}
		
		my $hundle = $cgi->upload("file");
		my $uploadfile = $wiki->config('attach_dir')."/".&Util::url_encode($pagename).".".&Util::url_encode($filename);
		
		open(DATA,">$uploadfile") or die $!;
		binmode(DATA);
		while(read($hundle,$_,16384)){ print DATA $_; }
		close(DATA);
				
		# attachץ饰󤫤źդ줿
		if(defined($cgi->param("count"))){
			my @lines = split(/\n/,$wiki->get_page($pagename));
			my $flag = 0;
			my $form_count = 1;
			my $count=$cgi->param("count");
			my $content = "";
			foreach(@lines){
				if(index($_," ")==0||index($_,"\t")==0){
					$content .= $_."\n";
					next;
				}
				if(index($_,"{{attach}}")!=-1 && $flag==0){
					if($form_count==$count){
						$content = $content."{{ref ".$filename."}}\n";
						$flag = 1;
					} else {
						$form_count++;
					}
				}
				$content = $content.$_."\n";
			}
			if($flag==1){
				$wiki->save_page($pagename,$content);
			}
		}
		
		# εϿ
		&write_log($wiki,"UPLOAD",$pagename,$filename);
		
		$wiki->redirect($pagename);
		
	#-------------------------------------------------------
	# ǧ
	} elsif($cgi->param("CONFIRM") ne ""){
		my $file = $cgi->param("file");
		if($file eq ""){
			return $wiki->error("ե뤬ꤵƤޤ");
		}
		
		my $buf = "";
		
		$buf .= "<a href=\"".$wiki->config('script_name')."?page=".&Util::url_encode($pagename)."\">".
		        $cgi->escapeHTML($pagename)."</a>".$cgi->escapeHTML($file)."ƤǤ\n".
		        "<form action=\"".$wiki->config('script_name')."\" method=\"POST\">\n".
		        "  <input type=\"submit\" name=\"DELETE\" value=\" \">".
		        "  <input type=\"hidden\" name=\"action\" value=\"ATTACH\">".
		        "  <input type=\"hidden\" name=\"page\" value=\"".$cgi->escapeHTML($pagename)."\">".
		        "  <input type=\"hidden\" name=\"file\" value=\"".$cgi->escapeHTML($file)."\">".
		        "</form>";
		return $buf;
	
	#-------------------------------------------------------
	# ¹
	} elsif($cgi->param("DELETE") ne ""){
		my $file = $cgi->param("file");
		if($file eq ""){
			return $wiki->error("ե뤬ꤵƤޤ");
		}
		
		# εϿ
		&write_log($wiki,"DELETE",$pagename,$file);

		unlink($wiki->config('attach_dir')."/".&Util::url_encode($pagename).".".&Util::url_encode($file));
		$wiki->redirect($pagename);
		
	#-------------------------------------------------------
	# 
	} else {
		my $file = $cgi->param("file");
		if($file eq ""){
			return $wiki->error("ե뤬ꤵƤޤ");
		}
		if(!$wiki->can_show($pagename)){
			return $wiki->error("ڡλȸ¤ޤ");
		}
		
		my $contenttype = &get_mime_type($wiki,$file);
		my $ua = $ENV{"HTTP_USER_AGENT"};
		my $disposition = ($contenttype =~ /^image\// && $ua !~ /MSIE/ ? "inline" : "attachment");
		print "Content-Type: $contenttype\n";
		print "Content-Disposition: $disposition;filename=\"".&Jcode::convert($file,'sjis')."\"\n\n";
		open(DATA,$wiki->config('attach_dir')."/".&Util::url_encode($pagename).".".&Util::url_encode($file)) or die $!;
		binmode(DATA);
		while(<DATA>){ print $_; }
		close(DATA);
				
		# εϿ
		&write_log($wiki,"DOWNLOAD",$pagename,$file);
		&count_up($wiki,$pagename,$file);
		
		exit();
	}
}

#===========================================================
# ɥȤ򥤥󥯥
#===========================================================
sub count_up {
	my $wiki = shift;
	my $page = shift;
	my $file = shift;
	
	my $count = Util::load_config_hash(undef,$wiki->config('log_dir')."/".$wiki->config('download_count_file'));
	unless(defined($count->{$page."::".$file})){
		$count->{$page."::".$file} = 1;
	} else {
		$count->{$page."::".$file}++;
	}
	Util::save_config_hash(undef, $wiki->config('log_dir')."/".$wiki->config('download_count_file'), $count);
}

#===========================================================
# źեեΥ
#===========================================================
sub write_log(){
	my $wiki = shift;
	my $mode = shift;
	my $page = shift;
	my $file = shift;
	if($wiki->config('log_dir') eq "" || $wiki->config('attach_log_file') eq ""){
		return;
	}
	my $ip  = $ENV{"REMOTE_ADDR"};
	my $ref = $ENV{"HTTP_REFERER"};
	my $ua  = $ENV{"HTTP_USER_AGENT"};
	if($ip  eq ""){ $ip  = "-"; }
	if($ref eq ""){ $ref = "-"; }
	if($ua  eq ""){ $ua  = "-"; }
	my ($sec, $min, $hour, $mday, $mon, $year) = localtime(time());
	my $date = sprintf("%04d/%02d/%02d %02d:%02d:%02d",$year+1900,$mon+1,$mday,$hour,$min,$sec);
	
	open(LOG,">>".$wiki->config('log_dir')."/".$wiki->config('attach_log_file')) or die $!;
	binmode(LOG);
	print LOG $mode." ".&Util::url_encode($page)." ".&Util::url_encode($file)." ".
	          $date." ".$ip." ".$ref." ".$ua."\n";
	close(LOG);
}

#===========================================================
# MIMEפޤ
#===========================================================
sub get_mime_type {
	my $wiki = shift;
	my $file = shift;
	my $type = lc(substr($file,rindex($file,".")+1));
	
	my $hash  = &Util::load_config_hash($wiki,"mime.dat");
	my $ctype = $hash->{$type};
	
	if($ctype eq "" ){
		$ctype = "application/octet-stream";
	}
	
	return $ctype;
}

1;
